/*
 * gpu_dwt.h
 *
 *  Created on: 16.12.2017
 *      Author: Piotr Romaniuk, ELESOFTROM
 */

#ifndef GPU_DWT_H_
#define GPU_DWT_H_

#include <stdint.h>

typedef uint32_t uniform_t;

#define MAX_QPU_USED 8

/** Structure passed uniforms to QPUs*/
struct qpu_uniforms{
	uniform_t dbg_rfiles;
	uniform_t dbg_accus;
	uniform_t filter;
	uniform_t qpu_number;
	uniform_t qpu_id;
	uniform_t v16_count_per_qpu;
	uniform_t in_data;
	uniform_t out_data;
};

#define INFO_RAW_SIZE 4096
#define PING_BUFFER_SIZE 4096
#define PONG_BUFFER_SIZE 4096
#define GPU_CODE_ALLOCATION 4096	//constant size, just to simplify memory image
#define GPU_DBG_RFILE_SIZE \
	    ( 4/*length of one SIMD element*/\
		*16 /*number of elements in SIMD*/\
		*(32+32)/*number of register: file ra +file rb*/)

#define GPU_DBG_ACCUS_SIZE \
		( 4/*length of one SIMD element*/\
		*16/*number of elements in SIMD*/\
		*4/*number of accumulators*/)

#define DWT_FILTER_SIZE  1280 /*TODO add exact calculation*/

/**@brief Memory image shared between GPU and CPU*/
struct gpu_memory_image{
	union {
		uint8_t raw[INFO_RAW_SIZE];
		struct GPU_FFT_BASE base;
		//struct GPU_FFT info;//this structure constains base at the begining
					//TODO clean it up!
		//...
	} info;
	struct{ //TODO rearange image, move this buffers to the end, it can grow
		uint8_t ping_buffer[PING_BUFFER_SIZE];//make this buffers variable length
		uint8_t pong_buffer[PONG_BUFFER_SIZE];
	} data;
	struct {
		uint8_t rfile[GPU_DBG_RFILE_SIZE];
		uint8_t accus[GPU_DBG_ACCUS_SIZE];
	} dbg;
	uint8_t code[GPU_CODE_ALLOCATION];//TODO add the test that checks if code fits here
	union {
		uint8_t raw[DWT_FILTER_SIZE];
		// indexes
		// coefficients
	} filter;
	struct qpu_uniforms uniforms[ MAX_QPU_USED ];

	struct { //when executed via mbox
		uint32_t uniforms;
		uint32_t code;
	}mbox_exe[ MAX_QPU_USED ];
};


/**@brief GPU memory descriptor*/
struct gpu_memory_dsc{
	uint32_t vc_ptr;/**<@brief VideoCore address of shared memory, use only in uniforms.
					Do not use as pointer in CPU program! */
	struct gpu_memory_image * image;/**<@brief image in CPU address space*/
};

/**@brief returns offset of the field in image*/
#define image_offset( field )  (uint32_t)&(((struct gpu_memory_image *)(void*)0)->field)

/**@brief Address of the field in Video Core address space - use for filing uniforms*/
#define vcptr_of_field( dsc, field )  (uint32_t)&(((struct gpu_memory_image *)(dsc)->vc_ptr)->field)

#define PRINT_PTR( txt_prefix, p_dsc, field ) \
		printf("%s: vc.ptr=%08x, arm.ptr=%p\n", txt_prefix, vcptr_of_field( p_dsc, field), &p_dsc->image-> field )

#endif /* GPU_DWT_H_ */
