/*
BCM2835 "GPU_FFT" release 3.0
Copyright (c) 2015, Andrew Holme.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the copyright holder nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __GPU_FFT__
#define __GPU_FFT__

#include "gpu_fft_base_dsc.h"
#include "gpu_dwt.h"


#define QPU_NUMBER 4

/**@brief Prepare to QPUs run (allocate memory, fill descriptors etc.)
 * @param[in] mb mailbox file_desc
 * @param[in] log2_N log2(FFT_length) = 8...22
 * @param[in] direction // GPU_FFT_FWD: fft(); GPU_FFT_REV: ifft()
 * @param[in] jobs   // number of transforms in batch
 * @param[in] gdsc GPU descriptor, images etc.
 * @param[in] n_simd16 length of input data [in number of 16 samples blocks]
 */
int gpu_prepare( int mb, int log2_N, int jobs, struct gpu_memory_dsc *gdsc, int n_simd16);

/**@brief start QPU execution
 * @param[in] base FFT descriptor
 */
unsigned gpu_execute(struct GPU_FFT_BASE *base);

/**@brief release allocated resources
* @param[in] base FFT descriptor
* */
void gpu_release(struct GPU_FFT_BASE *base);

//extern struct gpu_memory_dsc gdsc;

#endif // __GPU_FFT__
