/*
 * gpu_fft_base.h
 *
 *  Created on: 05.12.2017
 *      Author: Piotr Romaniuk, ELESOFTROM
 */

#ifndef GPU_FFT_BASE_H_
#define GPU_FFT_BASE_H_

#include "gpu_fft_base_dsc.h"
#include "gpu_dwt.h"

/**@brief general pointer description*/
struct GPU_FFT_PTR {
    unsigned vc;/**@brief pointer in Video Core address space */
    union {
    	void                   *vptr;
        char                   *bptr;
        float                  *fptr;
        unsigned               *uptr;
    } arm;//different types of pointers to memory block (from arm point of view)
};

/**@brief executes QPU code
 *
 * It can run QPU code in two ways:
 * 1) via mbox (base->vc_msg <>0)
 * 2) by direct register poking (base->vc_msg==0), with busy wait polling
 * @param[in] base QPU info
 * @param[in] num_qpus number QPUs (used only for [2])
 * @return 0 - Ok or 0x80000000 for timeout in [1] case
 */
unsigned gpu_fft_base_exec( struct GPU_FFT_BASE *base, int num_qpus);

/**@brief This function allocates memory and fills up ptr structure
 * @param[in] mb mbox id
 * @param[in] size size of allocated memory
 * @param[in] ptr structure for pointer description
 *            ptr->arm.vptr - contains GPU_FFT_BASE info (filled up)
 *
 * @return On success returns 0
 * @deprecated USe gpu_alloc instead
 */
int gpu_fft_alloc( int mb, unsigned size, struct GPU_FFT_PTR *ptr);

/**@brief This function allocates memory and fills dsc structure
 * @param[in] mb mbox id
 * @param[in] size size of allocated memory
 * @param[in] dsc structure for image description
 *            dsc->image->info.base - contains GPU_FFT_BASE info (filled up)
 *
 * @return On success returns 0
 */
int gpu_alloc ( int mb, unsigned size, struct gpu_memory_dsc * dsc);

/**@brief Releases allocated resources*/
void gpu_fft_base_release( struct GPU_FFT_BASE *base);

/**@brief Advances pointer ptr - two fields, for ARM and VC correspondigly*/
unsigned gpu_fft_ptr_inc ( struct GPU_FFT_PTR *ptr, int bytes);




#endif /* GPU_FFT_BASE_H_ */
