/*
 * gpu_fft_base_dsc.h
 *
 *  Created on: 05.12.2017
 *      Author: Piotr Romaniuk, ELESOFTROM
 */

#ifndef GPU_FFT_BASE_DSC_H_
#define GPU_FFT_BASE_DSC_H_

/**@brief number of QPUs used in one calculation */
#define GPU_FFT_QPUS 8

/**@brief base descriptor structure */
struct GPU_FFT_BASE {
    int mb;			/**<@brief mailbox device id*/
    unsigned handle,
		size,
		vc_msg,
		vc_code,
		vc_unifs[GPU_FFT_QPUS],
		peri_size;
    volatile unsigned *peri;/**<@brief peripherals mapped into memory (direct register access here ) */
};

// registers in GPU_FFT_BASE->peri[]:
// example of use: base->peri[V3D_L2CACTL] =...
//
// V3D spec: http://www.broadcom.com/docs/support/videocore/VideoCoreIV-AG100-R.pdf
#define V3D_L2CACTL (0xC00020>>2)
#define V3D_SLCACTL (0xC00024>>2)
#define V3D_SRQPC   (0xC00430>>2)
#define V3D_SRQUA   (0xC00434>>2)
#define V3D_SRQCS   (0xC0043c>>2)
#define V3D_DBCFG   (0xC00e00>>2)
#define V3D_DBQITE  (0xC00e2c>>2)
#define V3D_DBQITC  (0xC00e30>>2)

#endif /* GPU_FFT_BASE_DSC_H_ */
