/*

Basic code:    BCM2835 "GPU_FFT" release 3.0	   Copyright (c) 2015, Andrew Holme
Modifications: Discrete Wavelet Transform example, Copyright (c) 2017, Piotr Romaniuk, ELESOFTROM
(see QPU programming notes at: http://www.elesoftrom.com.pl/blog/en/ )

All rights reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the copyright holder nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "mailbox.h"
#include "gpu_fft.h"
#include "filter_db2.h"
#include "log.h"
#include "input-data.h"

char Usage[] =
	"Discrete wavelet transform\n"
    "Usage: dwt-test [loops]\n"
    "  loops  = number of test repeats, loops>0,       default 1\n";

unsigned Microseconds(void) {
    struct timespec ts;
    clock_gettime(CLOCK_REALTIME, &ts);
    return ts.tv_sec*1000000 + ts.tv_nsec/1000;
}



struct gpu_memory_dsc gdsc;

int main(int argc, char *argv[]) {
    int i, j, k, ret, loops, freq, log2_N = 8, jobs, mb = mbox_open();
    unsigned t[2];


    jobs   = 1;  // transforms per batch - not used
    loops  =  argc>1? atoi(argv[1]) : 1;  // test repetitions

    if ( jobs<1 || loops<1 ) {
        printf(Usage);
        return -1;
    }

    if( argc==2 )
    	if( strcmp(argv[1],"-h")==0 ||
    		strcmp(argv[1],"--help")==0 ){
    			printf(Usage);
    		        return -1;
    	}

    /*
     * Number of QPUs is set in gpu_fft.h (QPU_NUMBER)
     *  - shader adapts to that number (it is sent via uniforms)
     */

    /* TODO hardcoded data size in simd16 blocks per QPU*/
#define NUMBER_V16_PER_QPU 2
    printf("\nDiscrete wavelet transform\n");
    printf("\nNumber of QPUs=%d  Number of v16/QPU = %d\n", QPU_NUMBER, NUMBER_V16_PER_QPU);
    printf("\nInput data size=%d \n", sizeof(xdata)/sizeof(xdata[0]));

    ret = gpu_prepare(mb, log2_N, jobs, &gdsc, NUMBER_V16_PER_QPU); // call once


    switch(ret) {
        case -1: printf("Unable to enable V3D. "); return -1;
        case -2: printf("log2_N=%d not supported.  Try between 8 and 22.\n", log2_N);         return -1;
        case -3: printf("Out of memory.  Try a smaller batch or increase GPU memory.\n");     return -1;
        case -4: printf("Unable to map Videocore peripherals into ARM memory space.\n");      return -1;
        case -5: printf("Can't open libbcm_host.\n");                                         return -1;
    }

    for (k=0; k<loops; k++) {//multiple tests of the same calculations
    	int ix;
#if 0
        for(ix=0; ix<4096; ix++){	//filling buffers with marker, just to see what has been changed
        	gdsc.image->data.ping_buffer[ix] = 0xde;
        	gdsc.image->data.pong_buffer[ix+4096] = 0xb0;//second buffer
        }
#endif

        unsigned char * ping = gdsc.image->data.ping_buffer;
        unsigned char * pong = gdsc.image->data.pong_buffer;

        float * ping_f = (float*)ping;
        //load data to input buffer
        for(ix=0; ix<sizeof(xdata)/sizeof(xdata[0]); ix++){
        	ping_f[ix] = xdata[ix];
        }

#define DATA_OUT_SIZE_BYTES  NUMBER_V16_PER_QPU*QPU_NUMBER*16/*number of elems in vector*/*4/*size of one elems in bytes*/

        mem_dump_fi("","\nping (input data)", ping, DATA_OUT_SIZE_BYTES );//TODO remove hardcoded size
//TODO remove hardcoded input and output data, counter etc. (NUMBER_V16_PER_QPU)

        filter_load( gdsc.image->filter.raw );
        mem_dump_fi("", "\nFilter data", gdsc.image->filter.raw, 1280 );//filter data 1152 (will fit into shared+8*unique=1280
        					//TODO remove hardcoded filter data length

        usleep(1); // Yield to OS
        t[0] = Microseconds();
        gpu_execute(&gdsc.image->info.base); // call one or many times QPU execution
        t[1] = Microseconds();

        regs_dump( "\n>>> QPU REGISTERS (accus) ", gdsc.image->dbg.accus, 4, 1);
        regs_dump( "\n>>> QPU REGISTERS (r-file)", gdsc.image->dbg.rfile, 64, 0);




        mem_dump_fi("",  "\n>>> PONG buf (out data)", pong, DATA_OUT_SIZE_BYTES);//results of DWT


        printf("usecs = %d k=%d\n ", t[1]-t[0], k );

    }

    gpu_release(&gdsc.image->info.base); // Videocore memory lost if not freed !
    return 0;
}
