/*
 * log.c
 *
 *  Created on: 16.12.2017
 *      Author: Piotr Romaniuk, ELESOFTROM
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>

void mem_dump(const char *prefix, const char * text, const unsigned char * cptr, int len)
{
	int ix, offs;
	printf("%s\n", text);
	for(ix=0; ix<len; ix+=64){
		printf("%s %p:", prefix, cptr+ix );
	    for(offs=0; offs<64; offs+=4){
	       	printf("%08lx ", *(unsigned long*)&cptr[ix+offs] );
	    }
	    printf("\n");
	}
}

#define hex 0
#define flo 1
#define dec 2
#define dec8 3

struct _reg_dsc{
	//const char *regname;
	const char *varname;
	int format;
};

//Variables assignment to registers (these names will be printed when registers are dumped)
struct _reg_dsc reg_dsc[]=
	{ /*ra_00*/{"link0", hex }, /*ra_01*/{"vpm_conf", hex },
	  /*ra_02*/{"c_M4", flo }, /*ra_03*/{"c_L8", flo },
	  /*ra_04*/{"IApk", dec8 },/*ra_05*/{"IBpk", dec8 },/*ra_06*/{"ICpk", dec8 },/*ra_07*/{"save_proc", hex },
	  /*ra_08*/{"sync_proc", hex },/*ra_09*/{"--", hex },/*ra_10*/{"F1", flo },/*ra_11*/{"F3", flo },
	  /*ra_12*/{"F5", flo },/*ra_13*/{"F7", flo },/*ra_14*/{"F9", flo },

	  /*ra_15*/{"X0", flo },/*ra_16*/{"X2", flo },/*ra_17*/{"X4", flo },/*ra_18*/{"X6", flo },
	  /*ra_19*/{"X8", flo },/*ra_20*/{"X10", flo },/*ra_21*/{"Y1", flo },/*ra_22*/{"Y3", flo },
	  /*ra_23*/{"Y5", flo },/*ra_24*/{"Y7", flo },/*ra_25*/{"Y9", flo },

	  /*ra_26*/{"sum0", flo },/*ra_27*/{"strade", flo },/*ra_28*/{"--", flo },
	  /*ra_29*/{"0x40", hex},/*ra_30*/{"out", hex}, /*ra_31*/{"cnt",hex},

	 /*rb_00*/{"filter", hex }, /*rb_01*/{"indt", hex },
	 /*rb_02*/{"c_H12", flo }, /*rb_03*/{"c_L4", flo },
	 /*rb_04*/{"qpu_numb", hex },/*rb_05*/{"--", hex },/*rb_06*/{"save_slave", hex },/*rb_07*/{"sync_slave", hex },
	 /*rb_08*/{"qpu_id", hex },/*rb_09*/{"F0", flo },/*rb_10*/{"F2", flo },/*rb_11*/{"F4", flo },
	 /*rb_12*/{"F6", flo },/*rb_13*/{"F8", flo },/*rb_14*/{"F10", flo },

	 /*rb_15*/{"X1", flo },/*rb_16*/{"X3", flo },/*rb_17*/{"X5", flo },/*rb_18*/{"X7", flo },
 	 /*rb_19*/{"X9", flo },/*rb_20*/{"Y0", flo },/*rb_21*/{"Y2", flo },/*rb_22*/{"Y4", flo },
 	 /*rb_23*/{"Y6", flo },/*rb_24*/{"Y8", flo },/*rb_25*/{"Y10", flo },

	 /*rb_26*/{"--", flo },/*rb_27*/{"--", flo },/*rb_28*/{"--", flo },
	 /*rb_29*/{"--", hex},/*rb_30*/{"dbg_rfile", hex}, /*rb_31*/{"dbg_accus",hex}
	};


void regs_dump( const char * text, const char * cptr, int rlen, int are_accus)
{
	int ix, offs;
	int len = rlen*64;
	printf("%s %p\n", text, cptr);
	printf("SIMD16:");
	for(ix=0;ix<16;ix++){
		if(ix<10) printf(" ");
		printf("  % 2d    ",ix);
	}
	printf("\n");
	if( !are_accus )
	for(ix=0; ix<len; ix+=64){
		if( !are_accus ) printf("r%c_%02d %s: ", (ix<64*32)? 'a':'b', (ix/64)%32 , reg_dsc[ix/64].varname);
		else printf("r_%d  : ", (ix/64));
	    for(offs=0; offs<64; offs+=4){
	    	//if( offs%4==0 ) printf(" ");
	    	if( !are_accus ){
	    		float v;
				switch( reg_dsc[ix/64].format ){
				case flo:
					v = *(float*)&cptr[ix+offs];
					if( fabs(v)<100)
						printf("% 8.5f ", v );
					else
						printf("% 8.1e ", v );
					break;
				case hex:   printf("%08lx ", *(unsigned long*)&cptr[ix+offs] );break;
				case dec:   printf("%08ld ", *(unsigned long*)&cptr[ix+offs] );break;
				case dec8:
					  printf("%d %d %d %d, ", cptr[ix+0+offs], cptr[ix+1+offs], cptr[ix+2+offs], cptr[ix+3+offs]);
					  break;
				}
	    	}
	    }
	    printf("\n");
	}
	else
	{
		for(ix=0; ix<len; ix+=64){
			printf("r_%d  : ", (ix/64));
			for(offs=0; offs<64; offs+=4){
				float v = *(float*)&cptr[ix+offs];
				if( fabs(v)<100)
					printf("% 8.5f ", v );
				else
					printf("% 8.1e ", v );
			}
			printf("\n");
			}
		for(ix=0; ix<len; ix+=64){
			printf("r_%d  : ", (ix/64));
			for(offs=0; offs<64; offs+=4)
				printf("%08lx ", *(unsigned long*)&cptr[ix+offs] );
			printf("\n");
			}
	}
}

void mem_dump_float(const char * text, const char * cptr, int len)
{
	int ix, offs;
	printf("%s\n", text);
	for(ix=0; ix<len; ix+=64){
		printf("%-3d: ", (ix));
	    for(offs=0; offs<64; offs+=4){
	    	float * pf = (float*)&cptr[ix+offs];
	    	printf("% 8.5f ", *pf);
	    }
	    printf("\n");
	}
}


void mem_dump_fi(const char *prefix, const char * text, const unsigned char * cptr, int len)
{
	mem_dump_float(   text, cptr, len);
	mem_dump( prefix, text, cptr, len);
}

