/*
Copyright (c) 2012, Broadcom Europe Ltd.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the copyright holder nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MAILBOX_H__
#define __MAILBOX_H__



//========================= BCM DRIVER API ===================================
//  The driver provides V3D control from user space. Here is the API.
//============================================================================

/**@brief  open a char device file used for communicating with kernel mbox driver
 * @return mbox file id*/
int mbox_open();

/**@brief closes mbox
 * @param[in] file_desc mbox id
 */
void mbox_close(int file_desc);


/**@brief deprecated - NOT USED, NOR DEFINED!
 * @param[in] file_desc mbox id
 */
//unsigned get_version(int file_desc);

/**@brief memory allocation
 * @param[in] file_desc mbox id
 * @param[in] size
 * @param[in] align
 * @param[in] flags
 * @return handle?
 * */
unsigned mem_alloc(int file_desc, unsigned size, unsigned align, unsigned flags);

/**@brief memory free
 * @param[in] file_desc mbox id
 * @param[in] handle
 * @return ?
 */
unsigned mem_free(int file_desc, unsigned handle);

/**@brief memory lock
 * @param[in] file_desc mbox id
 * @param[in] handle
 * @return ?
 */
unsigned mem_lock(int file_desc, unsigned handle);

/**@brief memory unlock
 * @param[in] file_desc mbox id
 * @param[in] handle
 * @return ?
 */
unsigned mem_unlock(int file_desc, unsigned handle);

/**@brief executes short QPU code
 * @param[in] file_desc mbox id
 * @param[in] code
 * @param[in] r0
 * @param[in] r1
 * @param[in] r2
 * @param[in] r3
 * @param[in] r4
 * @param[in] r5
 * @return ?
 */
unsigned execute_code(int file_desc, unsigned code, unsigned r0, unsigned r1, unsigned r2, unsigned r3, unsigned r4, unsigned r5);

/**@brief executes QPU code
 * @param[in] file_desc mbox id
 * @param[in] num_qpus number of QPUs
 * @param[in] control
 * @param[in] noflush
 * @param[in] timeout
 */
unsigned execute_qpu(int file_desc, unsigned num_qpus, unsigned control, unsigned noflush, unsigned timeout);

/**@brief enable/disable QPU
 * @param[in] file_desc mbox id
 * @param[in] enable (0/1)
 */
unsigned qpu_enable(int file_desc, unsigned enable);



#endif /* __MAILBOX_H__ */
