#
# Discrete Wavelet Transform
#
# wavelet type: db2
# levels: 3 (d1,d2,d3,c3)
#
# Input data: 4*16 samples + 16 (padding wih zeros)
# Output data: 4*16 DWT coefficients, for each 16 samples: d1[8].d2[4].d3[2].c3[2]
#
# Copyright (c) 2017 Piotr Romaniuk, ELESOFTROM.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



.include "vc4.qinc"
.include "communication.qinc"
.include "regs_assign.qinc"
.include "breakpoint.qinc"

##############################################################################
#.set NUMBER_QPUS, 4
#.set STRADE, (64*NUMBER_QPUS)
.set NUMBER_OF_V16_RETURNED, 8 #TODO hardcoded, but should be determined from (counter x NUMBER_QPUs)

        mov     ra_0x40, 0x40 
        
##############################################################################
        mov dbg_rfile_dump, unif #buffer for r-files dump
        mov dbg_accus_dump, unif #buffer for accus dump
        mov filter, unif    # address of filter data
        mov qpu_number, unif # number of QPU used        
        mov qpu_id,  unif   # QPUid 
        mov counter, unif   # number of blocks (each block is 16 samples long)       
        mov in_data, unif   # input data buffer
        mov out_data, unif  # output data buffer
        
        mov r0, qpu_number
        shl ra_strade, r0, 6
        
        vpm_setup_prepare vpm_conf   # configure write to VPM location, different for each QPU
        
        # in_data
        #  |   vector16   |    vector16   |  | vector16   | vector 16
        #      QPU0           QPU1        ...    QPU0           QPU1
        #
        
        mov r1, vdw_setup_0(0, 16, dma_h32( 0,0))
        mul24 r0, counter, qpu_number
        shl r0, r0, 23
        add rb_vdw_conf, r0, r1
# Dynamic procedure address determination
# Bacause the code should be relocable, and is dependent on qpu_id procedures must be determined at run-time     
        
        get_subroutine_addr save_proc, r:1f #dynamical determination address of save_proc
#Subroutine that saves data for Master (QPU0)

        MUTEX_LOCK_VPM        
        mov vw_setup, vpm_conf       
        mov vpm, r0
        mov -, vw_wait         
        MUTEX_UNLOCK_VPM

        bra -, ra_link_0        #TODO may be optimized - qpu pipeline (but MUTEX_UNLOCK may be empty)
        nop
        nop
        nop
        
        
:1      
        get_subroutine_addr save_slave_proc, r:1f
#Subroutine that saves data for slaves (QPUn), n<>0
        
        MUTEX_LOCK_VPM        
        mov vw_setup, vpm_conf
        mov vpm, r0
        mov -, vw_wait 
        MUTEX_UNLOCK_VPM         #TODO may be optimized - qpu pipeline (but MUTEX_UNLOCK may be empty)
      
        bra -, ra_link_0
        nop
        nop
        nop
        
        
:1
        get_subroutine_addr sync_proc, r:1f
#Subroutine that synchronizes cores for master (QPU0)                
        .rep i, 7
        sacq -, 7  
        .endr
        bra -, ra_link_0         #TODO may be optimized (but depends on number of sacq above)
        nop
        nop
        nop
:1
       get_subroutine_addr sync_slave_proc, r:1f
#Subroutine that synchronizes cores for slaves (QPUn), n<>0
        bra -, ra_link_0
        srel -, 7        
        nop
        nop
:1
        #Determine save and sync procedures for each QPUid        
        mov.setf r0, qpu_id        
        mov.ifnz save_proc, save_slave_proc # different save procedure for master and slaves        
        mov.ifnz sync_proc, sync_slave_proc # the smae for final synchronization
        
        #Number of synchronized QPUs that Master will wait
        mov r0, qpu_number
        sub r0, 8, r0
        shl r0, r0, 3
        add.ifz sync_proc, sync_proc, r0
        
        
        #determine constants
        mov r0, 0.0 ; mov r1, 0.0        
        and.setf -, elem_num, 12            # z-flag only for 1st quad
        mov.ifz r0, 1.0 ; mov.ifnz r1, 1.0  # r0=[1111,0000,0000,0000], r1=[0000,1111,1111,1111]
        nop
        mov r2, r1<<4                           # r2=[1111,1111,1111,0000]        
        mov rb_const_H12, r2 ; mov r2, r0 >> 12 # r2=[0000,0000,0000,1111]
        mov rb_const_L4, r2 ; mov r2, r0 >>8    # r2=[0000,0000,1111,0000]        
        mov ra_const_M4, r2 ; mov r1, 0.0
        add.setf -, rb_const_L4, r2  
        mov.ifnz r1, 1.0                        #0000,0000,1111,1111
        mov ra_const_L8, r1
              
        # Read filter description (coefficients, indeces)
        mov r0, elem_num   # consecutive indexes (0..15)
        shl r0, r0, 2      # must be multiplied by 4, to get consecutive float values
        mov r1, filter     # where read coeficients from
   #    MUTEX_LOCK
        get_simd16_set_seq setI,  r1, r0, 3    #load packed indeces for data interleaved readout
        get_simd16_iset_seq F0, F1, r1, r0, 11 #load set of filter coefficients F0-F10
   #    MUTEX_UNLOCK
   
        # TMU0 requires values x4 for consecutive read of samples
        # indexes are packed so in each elements are 4 indexes
        #  each of four 8bit parts in each 32 bits element are multiplied by 4        
        v8adds r0, IA, IA  
        v8adds r1, IB, IB ; v8adds IA, r0, r0  
        v8adds r2, IC, IC ; v8adds IB, r1, r1 
        v8adds IC, r2, r2
         
        # start of data for each QPU 
        mov r0, in_data 
        mov r1, qpu_id        
        shl r1, r1, 6   #each QPU gets data with 64 bytes shift refering to previous QPU
        add r0, r0, r1        
        mov in_data, r0
        nop
        
        
:loop        
        mov r0, in_data
        
        MUTEX_LOCK
        load X0, r0, I0 #load interleaved data according to indeces in I0-I10
        load X1, r0, I1
        load X2, r0, I2
        load X3, r0, I3
        load X4, r0, I4
        load X5, r0, I5
        load X6, r0, I6
        load X7, r0, I7
        load X8, r0, I8
        load X9, r0, I9
        load X10, r0, I10
        MUTEX_UNLOCK
        
        # make parts of filtration: Yn = Xn * Fn, n=0...10
        # and then calculate DWT by:
        #   ya = sum(Y0...Y4), yb = sum(Y5...Y10)
        # then shift and sum properly parts in ya and yb
        
        fmul r0, X0, F0 #r0=Y0
        fmul r1, X1, F1 #r1=Y1
        fadd r0, r0, r1 ; fmul r1, X2, F2 #r0=Y0+Y1; r1=Y2
        fadd r0, r0, r1 ; fmul r1, X3, F3 #r0=Y0+Y1+Y2; r1=Y3       
        fadd r0, r0, r1 ; fmul r1, X4, F4 #r0=Y0+Y1+Y2+Y3; r1=Y4
        fadd r0, r0, r1 ; fmul r1, X5, F5 #r0=Y0+Y1+Y2+Y3+Y4=ya, r1=Y5
                          fmul r2, X6, F6 #r2=Y6
        fadd r1, r1, r2 ; fmul r2, X7, F7 #r1=Y5+Y6
        fadd r1, r1, r2 ; fmul r2, X8, F8 #r1=Y5+Y6+Y7 
        fadd r1, r1, r2 ; fmul r2, X9, F9 #r1=Y5+Y6+Y7+Y8        
     
        fadd r1, r1, r2 ; fmul r2, ra_const_M4, r0 << 4 
                                                
        fadd r0, r0, r2 ; fmul r3, X10, F10  # r0 = dwt.d1,d2-, 
        fadd r1, r1, r3 ; fmul ra_sum0, rb_const_H12, r0 
             # r1 = Y5+Y6+Y7+Y8+Y9+Y10=yb         
             # ra_sum0 = DWT.d1-d2-0000
        nop  # NOP may be deleted because only higher quads are important, left to keep compiler not complaining
                          # r1 = |a|b|c |d | - 4 quads
        mov  r2, r1 >>8   # r2:= |-|-|a |b |              
        fadd r0, r2, r1   # r0:= |-|-|ac|bd|
        
        nop  # NOP may be deleted because only higher quads are important, left to keep compiler not complaining
        mov  r2, r0 >>4   # r2:= |-|-|-|ac|      
        fadd r1, r2, r0   # r1:= |-|-|-|abcd|
        fmul r3, rb_const_L4, r1
        
        fadd r0, r3, ra_sum0 #r0-result DWT
        
        bra.setf ra_link_0, save_proc #Warning: qasm/gpu_test1.qasm (230,35): Using an odd register number as branch target implies .setf. Use explicit .setf to avoid this warning.
         nop   #may be optimized 
         nop
         nop
       
        add in_data, in_data, ra_strade   # move input pointer
        add vpm_conf, vpm_conf, qpu_number           
        sub.setf counter, counter, 1    # decrement counter of simd16 vectors to be processed                      
        brr.allnz -, r:loop
         nop
         nop
         nop
         
        bra ra_link_0, sync_proc #code variation:
                #slave:  signal to master that data has been written
                #master: wait for all slaves to finish write
        
         nop
         nop
         nop
        
        mov.setf -, qpu_id
        brr.allz -, r:master_end
         nop
         nop
         nop
        
        exit 0
        nop
        nop
        
:master_end        
        mov vw_setup, rb_vdw_conf   
        mov vw_setup, vdw_setup_1(0) # setup extra stride=0
        mov vw_addr, out_data  # set write address and start transfer
        mov -, vw_wait          # wait for complete the transfer
        BP # dump registers for debuging purpose
        exit 1
        
        


        
:breakpoint_body
   _breakpoint
   exit 1
   
